@@ -23,6 +23,7 @@
 
 #include "suck_config.h"
 #include "both.h"
+#include "phrases.h"
 
 #ifdef TIMEOUT
 # if TIME_WITH_SYS_TIME
@@ -41,6 +42,15 @@
 #include <signal.h>
 #endif
 
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>		/* for aix */
+#endif
+
+
+/* internal function proto */
+void vprint_phrases(FILE *, const char *, va_list);
+void convert_nl(char *);
+
 /*-----------------------------------------------------*/
 /* get next number in string */
 char *number(char *sp, int *intPtr) {
@@ -81,7 +91,7 @@ struct hostent *get_hostent(const char *host) {
 	struct hostent *hi = NULL;
 
 	if(host==NULL) { 
-		error_log(ERRLOG_REPORT,"No hostname given !\r\n"); 
+		error_log(ERRLOG_REPORT,both_phrases[0], NULL); 
 	}
 	else {
 		c=*host;
@@ -101,34 +111,36 @@ int connect_to_nntphost(const char *host, struct hostent **hi, FILE *msgs, unsig
 	struct in_addr *aptr;
 	struct in_addr saddr;
 	struct sockaddr_in address;
+	char sport[10];
 	int sockfd = -1;
 
-	fprintf(msgs, "Using Port %hu\n", portnr);
+	sprintf(sport, "%hu", portnr);	/* cause print_phrases wants all strings */
+	print_phrases(msgs, both_phrases[1], sport, NULL);
 
 	/* Find the internet address of the NNTP server */
  	*hi = get_hostent(host);
  	if(*hi == NULL) {
-		error_log(ERRLOG_REPORT,"%s: ",host);
-  		MyPerror("Could not get host information");
+		error_log(ERRLOG_REPORT,"%v1%: ",host, NULL);
+  		MyPerror(both_phrases[2]);
  	}
 	else {
-		fprintf(msgs, "Official host name: %s\r\n", (*hi)->h_name);
+		print_phrases(msgs, both_phrases[3], (*hi)->h_name, NULL);
  		while((ptr = *((*hi)->h_aliases)) != NULL) {
-			fprintf(msgs, "Alias %s\r\n", ptr );
+			print_phrases(msgs, both_phrases[4], ptr, NULL );
 			(*hi)->h_aliases++;
 		}
  		if((*hi)->h_addrtype != AF_INET) {
-  			error_log(ERRLOG_REPORT,"Unsupported address type.\n" );
+  			error_log(ERRLOG_REPORT, both_phrases[5], NULL);
 		}
 		else {
 			while((aptr = (struct in_addr *)*((*hi)->h_addr_list)++) != NULL) {
 				saddr = *aptr;
-				fprintf(msgs, "Address: %s\r\n", inet_ntoa(*aptr));
+				print_phrases(msgs, both_phrases[17], inet_ntoa(*aptr), NULL);
 			}
 
 			/* Create a socket */
  			if((sockfd = socket( AF_INET, SOCK_STREAM, SOCKET_PROTOCOL)) == -1) {
-				MyPerror("Socket failed:");
+				MyPerror(both_phrases[6]);
 			}
 			else { 
 				address.sin_family = AF_INET;
@@ -137,12 +149,12 @@ int connect_to_nntphost(const char *host, struct hostent **hi, FILE *msgs, unsig
 
 				/* Establish a connection */
 				if(connect(sockfd, (struct sockaddr *)&address, sizeof address ) == -1) {
-					MyPerror("Connect failed:");
+					MyPerror(both_phrases[7]);
 					close(sockfd);
 					sockfd = -1;
 				}
 				else {
-					fprintf(msgs,"Connected to %s\r\n",(*hi)->h_name);
+					print_phrases(msgs,both_phrases[8], (*hi)->h_name, NULL);
 				}
 			}
 		}
@@ -176,6 +188,7 @@ void do_debug(const char *fmt, ...) {
 		fclose(fptr);
 	}
 }
+/*------------------------------------------------------------*/
 #endif
 /*------------------------------------------------------------*/
 void MyPerror(const char *message) {
@@ -187,9 +200,9 @@ void MyPerror(const char *message) {
 	/* and I need to route it to my errlog */
 	/* so I have to recreate perror's format */
 #ifdef HAVE_STRERROR
-	error_log(ERRLOG_REPORT, "%s: %s\n", message, strerror(errno));
+	error_log(ERRLOG_REPORT, "%v1%: %v2%\n", message, strerror(errno), NULL);
 #else
-	error_log(ERRLOG_REPORT, "%s: Errno %d\n", message, errno);
+	error_log(ERRLOG_REPORT, both_phrases[9], message, errno, NULL);
 #endif
 }	
 /*-----------------------------------------------------------*/
@@ -243,7 +256,7 @@ int sgetline(int fd, char **inbuf) {
 				i = recv(fd, eob, MAXLINLEN-len, 0);	/* get line */
 			}
 			else if(i == 0) {
-				error_log(ERRLOG_REPORT, "Socket error: Timed Out on Read\n");
+				error_log(ERRLOG_REPORT, both_phrases[10], NULL);
 			}      /* other errors will be handled down below */
 #else 
 			i = recv(fd, eob, MAXLINLEN-len, 0);	/* get line */
@@ -259,10 +272,10 @@ int sgetline(int fd, char **inbuf) {
 			if(i < 1) {
 				if(i == 0) {
 					/* No data read in either from recv or select timed out*/
-					error_log(ERRLOG_REPORT, "Socket error: No Data to Read\n");
+					error_log(ERRLOG_REPORT, both_phrases[11], NULL);
 				}
 				else {
-					MyPerror("Socket error");
+					MyPerror(both_phrases[12]);
 				}
 				ret = -1;
 			}
@@ -305,7 +318,23 @@ int sgetline(int fd, char **inbuf) {
 	}
 #ifdef DEBUG1
 	if(ret > 0) {
-		do_debug("\nRETURNING len %d: %s", ret, *inbuf);
+		int flag = FALSE;
+		char saveit = '\0';
+		/* change nl to something we can see */
+		if((*inbuf)[ret-1] == '\n') {
+			flag = TRUE;
+			(*inbuf)[ret-1] = '\\';
+			(*inbuf)[ret] = 'n';
+			saveit = (*inbuf)[ret+1];
+			(*inbuf)[ret+1] = '\0';
+		}
+		do_debug("\nRETURNING len %d: %s\n", ret, *inbuf);
+		/* put things back the way they were */
+		if(flag == TRUE) {
+			(*inbuf)[ret-1] = '\n';
+			(*inbuf)[ret] = '\0';
+			(*inbuf)[ret+1] = saveit;
+		}
 	}
 	else {
 		do_debug("\nRETURNING error, ret = %d", ret);
@@ -324,9 +353,9 @@ void signal_block(int action) {
 	switch(action) {
 		case MYSIGNAL_SETUP:	/* This must be called first */
 			sigemptyset(&blockers);
-			if(sigaddset(&blockers, MYSIGNAL) == -1) {
+			if(sigaddset(&blockers, MYSIGNAL) == -1 || sigaddset(&blockers,PAUSESIGNAL) == -1) {
 				do_block = FALSE;
-				error_log(ERRLOG_REPORT, "Unable to block signal %d\n", MYSIGNAL);
+				error_log(ERRLOG_REPORT, both_phrases[13], str_int(MYSIGNAL), NULL);
 			}
 			else {
 				do_block = TRUE;
@@ -334,7 +363,7 @@ void signal_block(int action) {
 			break;
 		case MYSIGNAL_ADDPIPE:	/* add SIGPIPE for killprg.c */
 			if(sigaddset(&blockers, SIGPIPE) == -1) {
-				error_log(ERRLOG_REPORT, "Unable to block SIGPIPE, if child dies, we will\n");
+				error_log(ERRLOG_REPORT, both_phrases[14], NULL);
 			}
 			break;
 		case MYSIGNAL_BLOCK:
@@ -375,8 +404,7 @@ void error_log(int mode, const char *fmt, ...) {
 		if(errfile[0] == '\0' || (fptr = fopen(errfile, "a")) == NULL) {
 			fptr = stderr;
 		}
-		vfprintf(fptr, fmt, args);
-
+		vprint_phrases(fptr, fmt, args);
 		if(fptr != stderr) {
 			fclose(fptr);
 		}
@@ -424,7 +452,7 @@ char **build_args(const char *fname, int *nrargs) {
 		do_debug("Counted %d args in %s\n", counter, fname);
 #endif  
 		if((args = calloc( counter, sizeof(char *))) == NULL) {
-			error_log(ERRLOG_REPORT, "Out of Memory reading %s, ignoring\n", fname);
+			error_log(ERRLOG_REPORT, both_phrases[15], fname, NULL);
 		}
 		else {
 			/* pass two  read em and alloc em*/
@@ -447,7 +475,7 @@ char **build_args(const char *fname, int *nrargs) {
 							len++;	/* find length of arg */
 						}
 						if((args[counter] = calloc( len+1, sizeof(char))) == NULL) {
-							error_log(ERRLOG_REPORT, "Out of Memory on arg .*%s, ignoring\n", len, linein[x]);
+							error_log(ERRLOG_REPORT, both_phrases[16], NULL);
 						}
 						else {
 							strncpy(args[counter], &linein[x], len);
@@ -479,7 +507,204 @@ void free_args(int argc, char *argv[]) {
 	}
 	free(argv);
 }
+/*--------------------------------------------------------------------------------*/
+/* These are used by the load_phrases() stuff					  */
+/* read_array() and do_a_phrase() don't use the stored phrases                    */
+/* since they are used when the phrases are being loaded. 			  */
+/*--------------------------------------------------------------------------------*/
+char **read_array(FILE *fpi, int nr, int save_yn) {
 
+	int i, retval = 0;
+	char **array = NULL;
+	char linein[MAXLINLEN+1];
 
+	if(save_yn == TRUE && (array = calloc( nr, sizeof(char *))) == NULL) {
+		error_log(ERRLOG_REPORT, "Out of memory reading phrases\n", NULL);
+	}
+	else {
+		for(i=0;retval == 0 && i < nr; i++) {
+			if(fgets(linein, MAXLINLEN, fpi) != NULL) {
+				if(save_yn == TRUE) {
+					if((array[i] = do_a_phrase(linein)) == NULL) {
+						retval = -1;
+					}
+				}
+			}
+		}
+	}
+
+	if(retval == -1) {
+		free_array(nr, array);
+		array = NULL;
+	}
+	return array;
+}
+/*-------------------------------------------------------------------------------*/
+void free_array(int n, char **arr) {
+	int i;
+	if(arr != NULL) {
+		for(i=0;i<n;i++) {
+			if(arr[i] != NULL) {
+				free(arr[i]);
+			}
+		}
+		free(arr);
+	}	
+}
+/*--------------------------------------------------------------------------------*/
+char *do_a_phrase(char linein[]) {
+
+	char *lineout;
+	int i, stt, end;
+	static char default_phrase[] = "";
 
 	
+	i = strlen(linein);
+	lineout = NULL;
+	
+	/* find start and finish of string */
+	for(stt = 0; stt != i && linein[stt] != '"'; stt++) {
+	}
+	for(end = i-1;  i!= 0 && linein[end] != '"'; end--) {
+	}
+	if(end <= stt) {
+		error_log(ERRLOG_REPORT, "Invalid line, %v1%, Ignoring\n", linein, NULL);
+		lineout = default_phrase;
+	}
+	else {
+		i = end - stt;
+		if((lineout = calloc(i, sizeof(char))) == NULL) {
+			error_log(ERRLOG_REPORT, "Out of Memory Reading Phrases\n", NULL);
+		}
+		else {
+			strncpy(lineout, &linein[stt+1], i-1);	/* put the string into place */
+			lineout[i-1] = '\0';			/* null terminate it */
+#ifdef DEBUG1
+			do_debug("Got phrase:%s:\n", lineout);
+#endif
+			convert_nl(lineout);
+		}
+	}
+	return lineout;
+}
+/*-----------------------------------------------------------------------------------*/
+void convert_nl(char *linein) {
+	/* go thru a string, and convert \r \t \n (2 chars) to actual control codes */
+	char c;
+	int i, x, len;
+
+	if(linein != NULL) {
+		len = strlen(linein);
+		for(i = 0; i < len ; i++ ) {
+			if(linein[i] == '\\') {
+				c = linein[i+1];
+				if(c == 'r' || c == 'n' || c == 't') {
+					switch(c) {
+					  case 'r':
+						c = '\r';
+						break;
+					  case 'n':
+						c = '\n';
+						break;
+					  case 't':
+						c = '\t';
+						break;
+					}
+					linein[i] = c;
+					for ( x = i + 1; x < len ; x++ ) {
+						linein[x] = linein[x+1];
+					}
+					len--;		
+				}
+			}
+		}
+	}				
+}
+/*-----------------------------------------------------------------------------------*/
+void print_phrases(FILE *fpout, const char *argstr, ... ){
+
+	va_list vargs;
+	
+	va_start(vargs, argstr);
+
+	vprint_phrases(fpout, argstr, vargs);
+
+	va_end(vargs);
+}
+
+/*-----------------------------------------------------------------------------------*/
+void vprint_phrases(FILE *fpout, const char *argstr, va_list vargs) {
+	/* this routine takes in argstr, parses it for args and prints everything */
+	/* out, the args may not be in order ("hi %v2% there %v1% guys")	  */  
+	/* args are %v#%.  The varargs are all char ptrs */
+
+	const char *ptr, *pnumber;
+	char *tptr, block[PHRASES_BLOCK_SIZE+1], *in[PHRASES_MAX_NR_VARS];
+	int len, varyn, vnr, nrvars;
+
+	/* create an array with our string pointers */
+	nrvars = 0;
+	tptr = va_arg(vargs, char *);
+	while(tptr != NULL && nrvars < PHRASES_MAX_NR_VARS) {
+		in[nrvars++] = tptr;
+		tptr = va_arg(vargs, char *);
+	}		
+
+	varyn = FALSE;
+	ptr = argstr;
+	if(ptr != NULL) {
+		while( *ptr != '\0' ) {
+			len = 0;
+			while(len < PHRASES_BLOCK_SIZE && *ptr != '\0') {
+				if(*ptr == PHRASES_SEPARATOR && *(ptr+1) == PHRASES_VAR_CHAR) {
+					pnumber = ptr+2;
+					while(isdigit(*pnumber)) {
+						pnumber++;
+					}
+					if(pnumber != ptr+2 && *pnumber == PHRASES_SEPARATOR) {
+						/* bingo, we match syntax %v1% handle the variables */
+						sscanf(ptr+2, "%d", &vnr);	/* get the number */
+						/* now print it */
+						if(vnr <= nrvars) {
+							varyn = TRUE;
+							/* first, print what's currently in the buffer */
+							block[len] = '\0';
+							fputs(block,fpout);
+							len = 0;
+							/* now print the variable */
+							/* -1 cause array starts at 0 vars at 1 */
+							fputs(in[vnr-1], fpout);
+							ptr = pnumber+1;	/* advance past var */
+						}
+							
+					}
+				}
+				if(varyn == FALSE) {
+					block[len++] = *ptr++;
+				}
+				else {
+					varyn = FALSE;		/* reset it */
+				}
+			}
+			if(len > 0 ) {
+				block[len] = '\0';	/* null terminate string */
+				fputs(block, fpout);
+			}
+		}
+	}
+}
+/*----------------------------------------------------------*/
+char *str_int(int nrin) {
+	
+	static char strings[PHRASES_MAX_NR_VARS][12];	/* lets pray ints don't get bigger than this */
+	static int which = 0;
+	
+	/* use a set of rotating buffers so can make multiple str_int calls */
+	if(++which == PHRASES_MAX_NR_VARS) {
+		which = 0;
+	}
+	
+	sprintf(strings[which], "%d", nrin);
+
+	return strings[which];
+}
