@@ -1,6 +1,8 @@
 #ifndef _SUCK_SUCK_H
 #define _SUCK_SUCK_H 1
 
+#include <stdio.h>	/* for FILE */
+
 /* link list structure one for each article */
 typedef struct LinkList {
 	struct LinkList *next;
@@ -26,7 +28,10 @@ typedef struct {
 	int batch;
 	int pause_time;
 	int pause_nrmsgs;
+	int sig_pause_time;
+	int sig_pause_nrmsgs;
 	int ms_kludge;
+	int killfile_log;
 	unsigned short int portnr;
 	long rnews_size;
 	FILE *msgs;
@@ -35,6 +40,7 @@ typedef struct {
 	const char *host;
 	const char *batchfile;
 	const char *status_file_name;
+	const char *phrases;
 } Master, *PMaster;
 
 int get_a_chunk(int, FILE *);
