@@ -1,7 +1,7 @@
 #ifndef _SUCK_TIMER_H
 #define _SUCK_TIMER_H 1
 
-const char *TimerFunc(int, long);
+void TimerFunc(int, long, FILE *);
 
 enum  timer_funcs { TIMER_START, TIMER_ADDBYTES, TIMER_DISPLAY, TIMER_TOTALS, TIMER_TIMEONLY };
 
